package com.dzeio.openhealth.services

import android.annotation.SuppressLint
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.os.Build
import android.util.Log
import androidx.core.app.NotificationCompat
import androidx.core.app.NotificationManagerCompat
import androidx.work.PeriodicWorkRequestBuilder
import androidx.work.WorkerParameters
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.MainActivity
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseService
import java.util.*
import java.util.concurrent.TimeUnit

class WaterReminderService(
    private val context: Context,
    params: WorkerParameters
) : BaseService(context, params) {

    companion object {
        const val TAG = "${Application.TAG}/WaterService"
        fun setup(context: Context) {
            schedule(
                TAG,
                PeriodicWorkRequestBuilder<WaterReminderService>(15, TimeUnit.MINUTES)
                    .addTag(TAG)
                    .build(),
                context
            )
        }
    }

    @SuppressLint("UnspecifiedImmutableFlag")
    override fun doWork(): Result {
        Log.d(TAG, "Ran! ${Date().toLocaleString()}")
        with(NotificationManagerCompat.from(context)) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                notify(
                    (Math.random() * 100).toInt(), NotificationCompat.Builder(context, "openhealth")
                        .setContentTitle("Did you drink?")
                        .setContentText("Drink now!")
                        .setSmallIcon(R.drawable.ic_logo_small)
                        .setPriority(NotificationCompat.PRIORITY_HIGH)
                        .setContentIntent(
                            PendingIntent.getActivity(
                                context,
                                0,
                                Intent(context, MainActivity::class.java).apply {
                                    flags = Intent.FLAG_ACTIVITY_NEW_TASK
                                },
                                PendingIntent.FLAG_IMMUTABLE
                            )
                        )
                        .build()
                )
            } else {
                notify(
                    (Math.random() * 100).toInt(), NotificationCompat.Builder(context, "openhealth")
                        .setContentTitle("Did you drink?")
                        .setContentText("Drink now!")
                        .setSmallIcon(R.drawable.ic_logo_small)
                        .setPriority(NotificationCompat.PRIORITY_HIGH)
                        .setContentIntent(
                            PendingIntent.getActivity(
                                context,
                                0,
                                Intent(context, MainActivity::class.java).apply {
                                    flags = Intent.FLAG_ACTIVITY_NEW_TASK
                                },
                                0
                            )
                        )
                        .build()
                )
            }
        }
        return Result.success()
    }


}