package com.dzeio.openhealth.ui.home

import android.animation.ValueAnimator
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.graphics.Rect
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.weight.Water
import com.dzeio.openhealth.databinding.FragmentHomeBinding
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.ui.weight.AddWeightDialog
import com.dzeio.openhealth.utils.BitmapUtils
import com.dzeio.openhealth.utils.DrawUtils
import com.github.mikephil.charting.components.AxisBase
import com.github.mikephil.charting.components.Description
import com.github.mikephil.charting.components.XAxis
import com.github.mikephil.charting.data.Entry
import com.github.mikephil.charting.data.LineData
import com.github.mikephil.charting.data.LineDataSet
import com.github.mikephil.charting.formatter.ValueFormatter
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import java.text.DateFormat
import java.text.SimpleDateFormat
import java.util.*
import kotlin.math.min

@AndroidEntryPoint
class HomeFragment : BaseFragment<HomeViewModel, FragmentHomeBinding>(HomeViewModel::class.java) {

    companion object {
        const val TAG = "HomeFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentHomeBinding
        get() = FragmentHomeBinding::inflate

    // private lateinit var fit: GoogleFit

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewModel.init()

        binding.addWeight.setOnClickListener {
            AddWeightDialog().show(requireActivity().supportFragmentManager, null)
        }

        binding.fragmentHomeWaterAdd.setOnClickListener { _ ->

            val water = viewModel.water.value

            if (water == null) {

                val w = Water()
                w.value = 200
                viewModel.updateWater(w)
            } else {
                water.value += 200
                viewModel.updateWater(water)
            }
        }

        binding.fragmentHomeWaterRemove.setOnClickListener { _ ->

            val water = viewModel.water.value

            if (water != null) {

                water.value -= 200
                if (water.value == 0) {
                    viewModel.deleteWater(water)
                } else {
                    viewModel.updateWater(water)
                }
            }
        }

        binding.fragmentHomeWaterRemove.setOnClickListener {
            lifecycleScope.launch {

                val item = viewModel.fetchTodayWater().first()
                Log.d(TAG, "Collected latest $it")
                if (item != null) {
                    item.value -= 200
                    if (item.value == 0) {
                        viewModel.deleteWater(item)
                    } else {
                        viewModel.updateWater(item)
                    }
                }

            }
        }

        binding.listWeight.setOnClickListener {
            Log.d("T", "Trying to move")

            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavListWeight())

//            requireActivity().supportFragmentManager.commit {
//                replace(R.id.nav_host_fragment_content_main, ListWeightFragment())
//                addToBackStack(null)
//            }
        }

        // Setup Graph
//        binding.weightGraph.apply {
//            gridLabelRenderer.labelFormatter = DateAsXAxisLabelFormatter(
//                requireContext(),
//                DateFormat.getDateInstance(DateFormat.MEDIUM)
//            )
//            gridLabelRenderer.numHorizontalLabels = 3 // only 4 because of the space
//            gridLabelRenderer.numVerticalLabels = 3
//            gridLabelRenderer.gridStyle = GridLabelRenderer.GridStyle.HORIZONTAL
//            viewport.isXAxisBoundsManual = true
//            viewport.isScalable = true
//            viewport.setScalableY(true)
//            viewport.isScrollable = true
//            viewport.setScrollableY(true)
//            gridLabelRenderer.setHumanRounding(false);
//            addSeries(serie)
//        }

    }

//    private val serie: LineGraphSeries<DataPoint> = LineGraphSeries()
//    private val entries = LineGraphSeries<DataPoint>()

    private fun updateGraph(list: List<Weight>) {
        if (list.isNotEmpty()) {
            val first = list[0].timestamp
            val last = list[list.size - 1].timestamp
            val weekFirst = last - 604800000
//            binding.weightGraph.viewport.setMinX(if (first > weekFirst) first else weekFirst)
//            binding.weightGraph.viewport.setMaxX(last);


            val entries = ArrayList<Entry>()
            for (item in list) {
                entries.add(Entry(item.timestamp.toFloat(), item.weight))
            }

            val dataSet = LineDataSet(entries, "Label")
//            binding.weightGraph.xAxis.axisMinimum = if (first > weekFirst) first.toFloat() else weekFirst.toFloat()
//            binding.weightGraph.setExtraOffsets(
//                if (first > weekFirst) first.toFloat() else weekFirst.toFloat(),
//                100f,
//                last.toFloat(),
//                0f
//            )
            binding.weightGraph.isAutoScaleMinMaxEnabled = true
            //binding.weightGraph.setvir
            binding.weightGraph.isDragEnabled = true
            binding.weightGraph.isScaleYEnabled = false
            binding.weightGraph.description = Description().apply { isEnabled = false }
            binding.weightGraph.isScaleXEnabled = true
            binding.weightGraph.setPinchZoom(false)
            binding.weightGraph.xAxis.setLabelCount(5, true)
            binding.weightGraph.xAxis.position = XAxis.XAxisPosition.BOTTOM
            binding.weightGraph.axisRight.isEnabled = false
            binding.weightGraph.axisRight.setDrawGridLines(false)
            binding.weightGraph.xAxis.valueFormatter = object : ValueFormatter() {
                override fun getAxisLabel(value: Float, axis: AxisBase?): String {
                    return SimpleDateFormat(
                        "yyyy-MM-dd",
                        Locale.getDefault()
                    ).format(Date(value.toLong()))
                    //return super.getAxisLabel(value, axis)
                }
            }
            binding.weightGraph.data = LineData(dataSet)
            binding.weightGraph.invalidate()
            binding.weightGraph.setVisibleXRange(
                (list[list.size - 1].timestamp - 604800000).toFloat(),
                list[list.size - 1].timestamp.toFloat()
            )
        }


//        binding.weightGraph.gridLabelRenderer.numHorizontalLabels = 3 // only 4 because of the space
//        binding.weightGraph.gridLabelRenderer.numVerticalLabels = 3
//
//        binding.weightGraph.viewport.maxXAxisSize = 604800000.0 * 2
////        binding.weightGraph.viewport.setMinimalViewport(0.0, 604800000.0, minValue.toDouble(), maxValue.toDouble())
//
//        serie.resetData(list
//            .map { DataPoint(Date(it.timestamp), it.weight.toDouble()) }
//            .toTypedArray())
//
//        binding.weightGraph.viewport.scrollToEnd()

    }

    override fun onStart() {
        super.onStart()

        lifecycleScope.launchWhenStarted {
            viewModel.fetchWeights().collectLatest {
                updateGraph(it)
            }
        }

        viewModel.water.observe(viewLifecycleOwner) {
            if (it != null) {
                updateWater(it.value)
            } else {
                updateWater(0)
            }
        }

        updateWater(0)

    }

    private fun updateWater(water: Int) {
        val oldValue = binding.fragmentHomeWaterCurrent.text.toString().replace("ml", "").toInt()
        binding.fragmentHomeWaterCurrent.text = "${water}ml"

        val graph = BitmapUtils.convertToMutable(
            requireContext(),
            BitmapFactory.decodeResource(resources, R.drawable.ellipse)
        )

        graph?.let { btmp ->
            ValueAnimator.ofFloat(min(oldValue.toFloat(), 1200f), min(water.toFloat(), 1200f))
                .apply {
                    duration = 300
                    addUpdateListener {
                        val canvas = Canvas(btmp)
                        DrawUtils.drawArc(
                            canvas,
                            100 * it.animatedValue as Float / 1200f,
                            MaterialColors.getColor(
                                requireView(),
                                com.google.android.material.R.attr.colorPrimary
                            )
                        )
                        canvas.save()
                        binding.background.setImageBitmap(graph)
                    }
                    start()
                }
        }
    }
}